# Copyright (c) 2014 Amazon.com, Inc. or its affiliates.  All Rights Reserved
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish, dis-
# tribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the fol-
# lowing conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#

import boto
from boto.connection import AWSQueryConnection
from boto.regioninfo import RegionInfo
from boto.exception import JSONResponseError
from boto.logs import exceptions
from boto.compat import json


class CloudWatchLogsConnection(AWSQueryConnection):
    """
    Amazon CloudWatch Logs Service API Reference
    This is the Amazon CloudWatch Logs API Reference . Amazon
    CloudWatch Logs is a managed service for real time monitoring and
    archival of application logs. This guide provides detailed
    information about Amazon CloudWatch Logs actions, data types,
    parameters, and errors. For detailed information about Amazon
    CloudWatch Logs features and their associated API calls, go to the
    `Amazon CloudWatch Logs Developer Guide`_.

    Use the following links to get started using the Amazon CloudWatch
    API Reference :


    + `Actions`_: An alphabetical list of all Amazon CloudWatch Logs
      actions.
    + `Data Types`_: An alphabetical list of all Amazon CloudWatch
      Logs data types.
    + `Common Parameters`_: Parameters that all Query actions can use.
    + `Common Errors`_: Client and server errors that all actions can
      return.
    + `Regions and Endpoints`_: Itemized regions and endpoints for all
      AWS products.


    In addition to using the Amazon CloudWatch Logs API, you can also
    use the following SDKs and third-party libraries to access Amazon
    CloudWatch Logs programmatically.


    + `AWS SDK for Java Documentation`_
    + `AWS SDK for .NET Documentation`_
    + `AWS SDK for PHP Documentation`_
    + `AWS SDK for Ruby Documentation`_


    Developers in the AWS developer community also provide their own
    libraries, which you can find at the following AWS developer
    centers:


    + `AWS Java Developer Center`_
    + `AWS PHP Developer Center`_
    + `AWS Python Developer Center`_
    + `AWS Ruby Developer Center`_
    + `AWS Windows and .NET Developer Center`_
    """
    APIVersion = "2014-03-28"
    DefaultRegionName = "us-east-1"
    DefaultRegionEndpoint = "logs.us-east-1.amazonaws.com"
    ServiceName = "CloudWatchLogs"
    TargetPrefix = "Logs_20140328"
    ResponseError = JSONResponseError

    _faults = {
        "LimitExceededException": exceptions.LimitExceededException,
        "DataAlreadyAcceptedException": exceptions.DataAlreadyAcceptedException,
        "ResourceInUseException": exceptions.ResourceInUseException,
        "ServiceUnavailableException": exceptions.ServiceUnavailableException,
        "InvalidParameterException": exceptions.InvalidParameterException,
        "ResourceNotFoundException": exceptions.ResourceNotFoundException,
        "ResourceAlreadyExistsException": exceptions.ResourceAlreadyExistsException,
        "OperationAbortedException": exceptions.OperationAbortedException,
        "InvalidSequenceTokenException": exceptions.InvalidSequenceTokenException,
    }

    def __init__(self, **kwargs):
        region = kwargs.pop('region', None)
        if not region:
            region = RegionInfo(self, self.DefaultRegionName,
                                self.DefaultRegionEndpoint)

        if 'host' not in kwargs or kwargs['host'] is None:
            kwargs['host'] = region.endpoint

        super(CloudWatchLogsConnection, self).__init__(**kwargs)
        self.region = region

    def _required_auth_capability(self):
        return ['hmac-v4']

    def create_log_group(self, log_group_name):
        """
        Creates a new log group with the specified name. The name of
        the log group must be unique within a region for an AWS
        account. You can create up to 100 log groups per account.

        You must use the following guidelines when naming a log group:

        + Log group names can be between 1 and 512 characters long.
        + Allowed characters are az, AZ, 09, '_' (underscore), '-'
          (hyphen), '/' (forward slash), and '.' (period).



        Log groups are created with a default retention of 14 days.
        The retention attribute allow you to configure the number of
        days you want to retain log events in the specified log group.
        See the `SetRetention` operation on how to modify the
        retention of your log groups.

        :type log_group_name: string
        :param log_group_name:

        """
        params = {'logGroupName': log_group_name, }
        return self.make_request(action='CreateLogGroup',
                                 body=json.dumps(params))

    def create_log_stream(self, log_group_name, log_stream_name):
        """
        Creates a new log stream in the specified log group. The name
        of the log stream must be unique within the log group. There
        is no limit on the number of log streams that can exist in a
        log group.

        You must use the following guidelines when naming a log
        stream:

        + Log stream names can be between 1 and 512 characters long.
        + The ':' colon character is not allowed.

        :type log_group_name: string
        :param log_group_name:

        :type log_stream_name: string
        :param log_stream_name:

        """
        params = {
            'logGroupName': log_group_name,
            'logStreamName': log_stream_name,
        }
        return self.make_request(action='CreateLogStream',
                                 body=json.dumps(params))

    def delete_log_group(self, log_group_name):
        """
        Deletes the log group with the specified name. Amazon
        CloudWatch Logs will delete a log group only if there are no
        log streams and no metric filters associated with the log
        group. If this condition is not satisfied, the request will
        fail and the log group will not be deleted.

        :type log_group_name: string
        :param log_group_name:

        """
        params = {'logGroupName': log_group_name, }
        return self.make_request(action='DeleteLogGroup',
                                 body=json.dumps(params))

    def delete_log_stream(self, log_group_name, log_stream_name):
        """
        Deletes a log stream and permanently deletes all the archived
        log events associated with it.

        :type log_group_name: string
        :param log_group_name:

        :type log_stream_name: string
        :param log_stream_name:

        """
        params = {
            'logGroupName': log_group_name,
            'logStreamName': log_stream_name,
        }
        return self.make_request(action='DeleteLogStream',
                                 body=json.dumps(params))

    def delete_metric_filter(self, log_group_name, filter_name):
        """
        Deletes a metric filter associated with the specified log
        group.

        :type log_group_name: string
        :param log_group_name:

        :type filter_name: string
        :param filter_name: The name of the metric filter.

        """
        params = {
            'logGroupName': log_group_name,
            'filterName': filter_name,
        }
        return self.make_request(action='DeleteMetricFilter',
                                 body=json.dumps(params))

    def delete_retention_policy(self, log_group_name):
        """
        

        :type log_group_name: string
        :param log_group_name:

        """
        params = {'logGroupName': log_group_name, }
        return self.make_request(action='DeleteRetentionPolicy',
                                 body=json.dumps(params))

    def describe_log_groups(self, log_group_name_prefix=None,
                            next_token=None, limit=None):
        """
        Returns all the log groups that are associated with the AWS
        account making the request. The list returned in the response
        is ASCII-sorted by log group name.

        By default, this operation returns up to 50 log groups. If
        there are more log groups to list, the response would contain
        a `nextToken` value in the response body. You can also limit
        the number of log groups returned in the response by
        specifying the `limit` parameter in the request.

        :type log_group_name_prefix: string
        :param log_group_name_prefix:

        :type next_token: string
        :param next_token: A string token used for pagination that points to
            the next page of results. It must be a value obtained from the
            response of the previous `DescribeLogGroups` request.

        :type limit: integer
        :param limit: The maximum number of items returned in the response. If
            you don't specify a value, the request would return up to 50 items.

        """
        params = {}
        if log_group_name_prefix is not None:
            params['logGroupNamePrefix'] = log_group_name_prefix
        if next_token is not None:
            params['nextToken'] = next_token
        if limit is not None:
            params['limit'] = limit
        return self.make_request(action='DescribeLogGroups',
                                 body=json.dumps(params))

    def describe_log_streams(self, log_group_name,
                             log_stream_name_prefix=None, next_token=None,
                             limit=None):
        """
        Returns all the log streams that are associated with the
        specified log group. The list returned in the response is
        ASCII-sorted by log stream name.

        By default, this operation returns up to 50 log streams. If
        there are more log streams to list, the response would contain
        a `nextToken` value in the response body. You can also limit
        the number of log streams returned in the response by
        specifying the `limit` parameter in the request.

        :type log_group_name: string
        :param log_group_name:

        :type log_stream_name_prefix: string
        :param log_stream_name_prefix:

        :type next_token: string
        :param next_token: A string token used for pagination that points to
            the next page of results. It must be a value obtained from the
            response of the previous `DescribeLogStreams` request.

        :type limit: integer
        :param limit: The maximum number of items returned in the response. If
            you don't specify a value, the request would return up to 50 items.

        """
        params = {'logGroupName': log_group_name, }
        if log_stream_name_prefix is not None:
            params['logStreamNamePrefix'] = log_stream_name_prefix
        if next_token is not None:
            params['nextToken'] = next_token
        if limit is not None:
            params['limit'] = limit
        return self.make_request(action='DescribeLogStreams',
                                 body=json.dumps(params))

    def describe_metric_filters(self, log_group_name,
                                filter_name_prefix=None, next_token=None,
                                limit=None):
        """
        Returns all the metrics filters associated with the specified
        log group. The list returned in the response is ASCII-sorted
        by filter name.

        By default, this operation returns up to 50 metric filters. If
        there are more metric filters to list, the response would
        contain a `nextToken` value in the response body. You can also
        limit the number of metric filters returned in the response by
        specifying the `limit` parameter in the request.

        :type log_group_name: string
        :param log_group_name:

        :type filter_name_prefix: string
        :param filter_name_prefix: The name of the metric filter.

        :type next_token: string
        :param next_token: A string token used for pagination that points to
            the next page of results. It must be a value obtained from the
            response of the previous `DescribeMetricFilters` request.

        :type limit: integer
        :param limit: The maximum number of items returned in the response. If
            you don't specify a value, the request would return up to 50 items.

        """
        params = {'logGroupName': log_group_name, }
        if filter_name_prefix is not None:
            params['filterNamePrefix'] = filter_name_prefix
        if next_token is not None:
            params['nextToken'] = next_token
        if limit is not None:
            params['limit'] = limit
        return self.make_request(action='DescribeMetricFilters',
                                 body=json.dumps(params))

    def get_log_events(self, log_group_name, log_stream_name,
                       start_time=None, end_time=None, next_token=None,
                       limit=None, start_from_head=None):
        """
        Retrieves log events from the specified log stream. You can
        provide an optional time range to filter the results on the
        event `timestamp`.

        By default, this operation returns as much log events as can
        fit in a response size of 1MB, up to 10,000 log events. The
        response will always include a `nextForwardToken` and a
        `nextBackwardToken` in the response body. You can use any of
        these tokens in subsequent `GetLogEvents` requests to paginate
        through events in either forward or backward direction. You
        can also limit the number of log events returned in the
        response by specifying the `limit` parameter in the request.

        :type log_group_name: string
        :param log_group_name:

        :type log_stream_name: string
        :param log_stream_name:

        :type start_time: long
        :param start_time: A point in time expressed as the number milliseconds
            since Jan 1, 1970 00:00:00 UTC.

        :type end_time: long
        :param end_time: A point in time expressed as the number milliseconds
            since Jan 1, 1970 00:00:00 UTC.

        :type next_token: string
        :param next_token: A string token used for pagination that points to
            the next page of results. It must be a value obtained from the
            `nextForwardToken` or `nextBackwardToken` fields in the response of
            the previous `GetLogEvents` request.

        :type limit: integer
        :param limit: The maximum number of log events returned in the
            response. If you don't specify a value, the request would return as
            much log events as can fit in a response size of 1MB, up to 10,000
            log events.

        :type start_from_head: boolean
        :param start_from_head:

        """
        params = {
            'logGroupName': log_group_name,
            'logStreamName': log_stream_name,
        }
        if start_time is not None:
            params['startTime'] = start_time
        if end_time is not None:
            params['endTime'] = end_time
        if next_token is not None:
            params['nextToken'] = next_token
        if limit is not None:
            params['limit'] = limit
        if start_from_head is not None:
            params['startFromHead'] = start_from_head
        return self.make_request(action='GetLogEvents',
                                 body=json.dumps(params))

    def put_log_events(self, log_group_name, log_stream_name, log_events,
                       sequence_token=None):
        """
        Uploads a batch of log events to the specified log stream.

        Every PutLogEvents request must include the `sequenceToken`
        obtained from the response of the previous request. An upload
        in a newly created log stream does not require a
        `sequenceToken`.

        The batch of events must satisfy the following constraints:

        + The maximum batch size is 32,768 bytes, and this size is
          calculated as the sum of all event messages in UTF-8, plus 26
          bytes for each log event.
        + None of the log events in the batch can be more than 2 hours
          in the future.
        + None of the log events in the batch can be older than 14
          days or the retention period of the log group.
        + The log events in the batch must be in chronological ordered
          by their `timestamp`.
        + The maximum number of log events in a batch is 1,000.

        :type log_group_name: string
        :param log_group_name:

        :type log_stream_name: string
        :param log_stream_name:

        :type log_events: list
        :param log_events: A list of events belonging to a log stream.

        :type sequence_token: string
        :param sequence_token: A string token that must be obtained from the
            response of the previous `PutLogEvents` request.

        """
        params = {
            'logGroupName': log_group_name,
            'logStreamName': log_stream_name,
            'logEvents': log_events,
        }
        if sequence_token is not None:
            params['sequenceToken'] = sequence_token
        return self.make_request(action='PutLogEvents',
                                 body=json.dumps(params))

    def put_metric_filter(self, log_group_name, filter_name, filter_pattern,
                          metric_transformations):
        """
        Creates or updates a metric filter and associates it with the
        specified log group. Metric filters allow you to configure
        rules to extract metric data from log events ingested through
        `PutLogEvents` requests.

        :type log_group_name: string
        :param log_group_name:

        :type filter_name: string
        :param filter_name: The name of the metric filter.

        :type filter_pattern: string
        :param filter_pattern:

        :type metric_transformations: list
        :param metric_transformations:

        """
        params = {
            'logGroupName': log_group_name,
            'filterName': filter_name,
            'filterPattern': filter_pattern,
            'metricTransformations': metric_transformations,
        }
        return self.make_request(action='PutMetricFilter',
                                 body=json.dumps(params))

    def put_retention_policy(self, log_group_name, retention_in_days):
        """
        

        :type log_group_name: string
        :param log_group_name:

        :type retention_in_days: integer
        :param retention_in_days: Specifies the number of days you want to
            retain log events in the specified log group. Possible values are:
            1, 3, 5, 7, 14, 30, 60, 90, 120, 150, 180, 365, 400, 547, 730.

        """
        params = {
            'logGroupName': log_group_name,
            'retentionInDays': retention_in_days,
        }
        return self.make_request(action='PutRetentionPolicy',
                                 body=json.dumps(params))

    def set_retention(self, log_group_name, retention_in_days):
        """
        Sets the retention of the specified log group. Log groups are
        created with a default retention of 14 days. The retention
        attribute allow you to configure the number of days you want
        to retain log events in the specified log group.

        :type log_group_name: string
        :param log_group_name:

        :type retention_in_days: integer
        :param retention_in_days: Specifies the number of days you want to
            retain log events in the specified log group. Possible values are:
            1, 3, 5, 7, 14, 30, 60, 90, 120, 150, 180, 365, 400, 547, 730.

        """
        params = {
            'logGroupName': log_group_name,
            'retentionInDays': retention_in_days,
        }
        return self.make_request(action='SetRetention',
                                 body=json.dumps(params))

    def test_metric_filter(self, filter_pattern, log_event_messages):
        """
        Tests the filter pattern of a metric filter against a sample
        of log event messages. You can use this operation to validate
        the correctness of a metric filter pattern.

        :type filter_pattern: string
        :param filter_pattern:

        :type log_event_messages: list
        :param log_event_messages:

        """
        params = {
            'filterPattern': filter_pattern,
            'logEventMessages': log_event_messages,
        }
        return self.make_request(action='TestMetricFilter',
                                 body=json.dumps(params))

    def make_request(self, action, body):
        headers = {
            'X-Amz-Target': '%s.%s' % (self.TargetPrefix, action),
            'Host': self.region.endpoint,
            'Content-Type': 'application/x-amz-json-1.1',
            'Content-Length': str(len(body)),
        }
        http_request = self.build_base_http_request(
            method='POST', path='/', auth_path='/', params={},
            headers=headers, data=body)
        response = self._mexe(http_request, sender=None,
                              override_num_retries=10)
        response_body = response.read().decode('utf-8')
        boto.log.debug(response_body)
        if response.status == 200:
            if response_body:
                return json.loads(response_body)
        else:
            json_body = json.loads(response_body)
            fault_name = json_body.get('__type', None)
            exception_class = self._faults.get(fault_name, self.ResponseError)
            raise exception_class(response.status, response.reason,
                                  body=json_body)



# Copyright (c) 2006-2009 Mitch Garnaat http://garnaat.org/
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish, dis-
# tribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the fol-
# lowing conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
from boto.resultset import ResultSet
from boto.ec2.ec2object import EC2Object
from boto.utils import parse_ts


class ReservedInstancesOffering(EC2Object):

    def __init__(self, connection=None, id=None, instance_type=None,
                 availability_zone=None, duration=None, fixed_price=None,
                 usage_price=None, description=None, instance_tenancy=None,
                 currency_code=None, offering_type=None,
                 recurring_charges=None, pricing_details=None):
        super(ReservedInstancesOffering, self).__init__(connection)
        self.id = id
        self.instance_type = instance_type
        self.availability_zone = availability_zone
        self.duration = duration
        self.fixed_price = fixed_price
        self.usage_price = usage_price
        self.description = description
        self.instance_tenancy = instance_tenancy
        self.currency_code = currency_code
        self.offering_type = offering_type
        self.recurring_charges = recurring_charges
        self.pricing_details = pricing_details

    def __repr__(self):
        return 'ReservedInstanceOffering:%s' % self.id

    def startElement(self, name, attrs, connection):
        if name == 'recurringCharges':
            self.recurring_charges = ResultSet([('item', RecurringCharge)])
            return self.recurring_charges
        elif name == 'pricingDetailsSet':
            self.pricing_details = ResultSet([('item', PricingDetail)])
            return self.pricing_details
        return None

    def endElement(self, name, value, connection):
        if name == 'reservedInstancesOfferingId':
            self.id = value
        elif name == 'instanceType':
            self.instance_type = value
        elif name == 'availabilityZone':
            self.availability_zone = value
        elif name == 'duration':
            self.duration = int(value)
        elif name == 'fixedPrice':
            self.fixed_price = value
        elif name == 'usagePrice':
            self.usage_price = value
        elif name == 'productDescription':
            self.description = value
        elif name == 'instanceTenancy':
            self.instance_tenancy = value
        elif name == 'currencyCode':
            self.currency_code = value
        elif name == 'offeringType':
            self.offering_type = value
        elif name == 'marketplace':
            self.marketplace = True if value == 'true' else False

    def describe(self):
        print('ID=%s' % self.id)
        print('\tInstance Type=%s' % self.instance_type)
        print('\tZone=%s' % self.availability_zone)
        print('\tDuration=%s' % self.duration)
        print('\tFixed Price=%s' % self.fixed_price)
        print('\tUsage Price=%s' % self.usage_price)
        print('\tDescription=%s' % self.description)

    def purchase(self, instance_count=1, dry_run=False):
        return self.connection.purchase_reserved_instance_offering(
            self.id,
            instance_count,
            dry_run=dry_run
        )


class RecurringCharge(object):
    def __init__(self, connection=None, frequency=None, amount=None):
        self.frequency = frequency
        self.amount = amount

    def startElement(self, name, attrs, connection):
        return None

    def endElement(self, name, value, connection):
        setattr(self, name, value)


class PricingDetail(object):
    def __init__(self, connection=None, price=None, count=None):
        self.price = price
        self.count = count

    def startElement(self, name, attrs, connection):
        return None

    def endElement(self, name, value, connection):
        setattr(self, name, value)


class ReservedInstance(ReservedInstancesOffering):

    def __init__(self, connection=None, id=None, instance_type=None,
                 availability_zone=None, duration=None, fixed_price=None,
                 usage_price=None, description=None,
                 instance_count=None, state=None):
        super(ReservedInstance, self).__init__(connection, id, instance_type,
                                               availability_zone, duration,
                                               fixed_price, usage_price,
                                               description)
        self.instance_count = instance_count
        self.state = state
        self.start = None
        self.end = None

    def __repr__(self):
        return 'ReservedInstance:%s' % self.id

    def endElement(self, name, value, connection):
        if name == 'reservedInstancesId':
            self.id = value
        if name == 'instanceCount':
            self.instance_count = int(value)
        elif name == 'state':
            self.state = value
        elif name == 'start':
            self.start = value
        elif name == 'end':
            self.end = value
        else:
            super(ReservedInstance, self).endElement(name, value, connection)


class ReservedInstanceListing(EC2Object):
    def __init__(self, connection=None, listing_id=None, id=None,
                 create_date=None, update_date=None,
                 status=None, status_message=None, client_token=None):
        self.connection = connection
        self.listing_id = listing_id
        self.id = id
        self.create_date = create_date
        self.update_date = update_date
        self.status = status
        self.status_message = status_message
        self.client_token = client_token

    def startElement(self, name, attrs, connection):
        if name == 'instanceCounts':
            self.instance_counts = ResultSet([('item', InstanceCount)])
            return self.instance_counts
        elif name == 'priceSchedules':
            self.price_schedules = ResultSet([('item', PriceSchedule)])
            return self.price_schedules
        return None

    def endElement(self, name, value, connection):
        if name == 'reservedInstancesListingId':
            self.listing_id = value
        elif name == 'reservedInstancesId':
            self.id = value
        elif name == 'createDate':
            self.create_date = value
        elif name == 'updateDate':
            self.update_date = value
        elif name == 'status':
            self.status = value
        elif name == 'statusMessage':
            self.status_message = value
        else:
            setattr(self, name, value)


class InstanceCount(object):
    def __init__(self, connection=None, state=None, instance_count=None):
        self.state = state
        self.instance_count = instance_count

    def startElement(self, name, attrs, connection):
        return None

    def endElement(self, name, value, connection):
        if name == 'state':
            self.state = value
        elif name == 'instanceCount':
            self.instance_count = int(value)
        else:
            setattr(self, name, value)


class PriceSchedule(object):
    def __init__(self, connection=None, term=None, price=None,
                 currency_code=None, active=None):
        self.connection = connection
        self.term = term
        self.price = price
        self.currency_code = currency_code
        self.active = active

    def startElement(self, name, attrs, connection):
        return None

    def endElement(self, name, value, connection):
        if name == 'term':
            self.term = int(value)
        elif name == 'price':
            self.price = value
        elif name == 'currencyCode':
            self.currency_code = value
        elif name == 'active':
            self.active = True if value == 'true' else False
        else:
            setattr(self, name, value)


class ReservedInstancesConfiguration(object):
    def __init__(self, connection=None, availability_zone=None, platform=None,
                 instance_count=None, instance_type=None):
        self.connection = connection
        self.availability_zone = availability_zone
        self.platform = platform
        self.instance_count = instance_count
        self.instance_type = instance_type

    def startElement(self, name, attrs, connection):
        return None

    def endElement(self, name, value, connection):
        if name == 'availabilityZone':
            self.availability_zone = value
        elif name == 'platform':
            self.platform = value
        elif name == 'instanceCount':
            self.instance_count = int(value)
        elif name == 'instanceType':
            self.instance_type = value
        else:
            setattr(self, name, value)


class ModifyReservedInstancesResult(object):
    def __init__(self, connection=None, modification_id=None):
        self.connection = connection
        self.modification_id = modification_id

    def startElement(self, name, attrs, connection):
        return None

    def endElement(self, name, value, connection):
        if name == 'reservedInstancesModificationId':
            self.modification_id = value
        else:
            setattr(self, name, value)


class ModificationResult(object):
    def __init__(self, connection=None, modification_id=None,
                 availability_zone=None, platform=None, instance_count=None,
                 instance_type=None):
        self.connection = connection
        self.modification_id = modification_id
        self.availability_zone = availability_zone
        self.platform = platform
        self.instance_count = instance_count
        self.instance_type = instance_type

    def startElement(self, name, attrs, connection):
        return None

    def endElement(self, name, value, connection):
        if name == 'reservedInstancesModificationId':
            self.modification_id = value
        elif name == 'availabilityZone':
            self.availability_zone = value
        elif name == 'platform':
            self.platform = value
        elif name == 'instanceCount':
            self.instance_count = int(value)
        elif name == 'instanceType':
            self.instance_type = value
        else:
            setattr(self, name, value)


class ReservedInstancesModification(object):
    def __init__(self, connection=None, modification_id=None,
                 reserved_instances=None, modification_results=None,
                 create_date=None, update_date=None, effective_date=None,
                 status=None, status_message=None, client_token=None):
        self.connection = connection
        self.modification_id = modification_id
        self.reserved_instances = reserved_instances
        self.modification_results = modification_results
        self.create_date = create_date
        self.update_date = update_date
        self.effective_date = effective_date
        self.status = status
        self.status_message = status_message
        self.client_token = client_token

    def startElement(self, name, attrs, connection):
        if name == 'reservedInstancesSet':
            self.reserved_instances = ResultSet([
                ('item', ReservedInstance)
            ])
            return self.reserved_instances
        elif name == 'modificationResultSet':
            self.modification_results = ResultSet([
                ('item', ModificationResult)
            ])
            return self.modification_results
        return None

    def endElement(self, name, value, connection):
        if name == 'reservedInstancesModificationId':
            self.modification_id = value
        elif name == 'createDate':
            self.create_date = parse_ts(value)
        elif name == 'updateDate':
            self.update_date = parse_ts(value)
        elif name == 'effectiveDate':
            self.effective_date = parse_ts(value)
        elif name == 'status':
            self.status = value
        elif name == 'statusMessage':
            self.status_message = value
        elif name == 'clientToken':
            self.client_token = value
        else:
            setattr(self, name, value)



"""
Exceptions that are specific to the cloudsearch2 module.
"""
from boto.exception import BotoServerError


class InvalidTypeException(BotoServerError):
    """
    Raised when an invalid record type is passed to CloudSearch.
    """
    pass


class LimitExceededException(BotoServerError):
    """
    Raised when a limit has been exceeded.
    """
    pass


class InternalException(BotoServerError):
    """
    A generic server-side error.
    """
    pass


class DisabledOperationException(BotoServerError):
    """
    Raised when an operation has been disabled.
    """
    pass


class ResourceNotFoundException(BotoServerError):
    """
    Raised when a requested resource does not exist.
    """
    pass


class BaseException(BotoServerError):
    """
    A generic server-side error.
    """
    pass



# Copyright (c) 202 Mitch Garnaat http://garnaat.org/
# Copyright (c) 2012 Amazon.com, Inc. or its affiliates.
# All Rights Reserved
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish, dis-
# tribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the fol-
# lowing conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

class SourceAttribute(object):
    """
    Provide information about attributes for an index field.
    A maximum of 20 source attributes can be configured for
    each index field.

    :ivar default: Optional default value if the source attribute
        is not specified in a document.
        
    :ivar name: The name of the document source field to add
        to this ``IndexField``.

    :ivar data_function: Identifies the transformation to apply
        when copying data from a source attribute.
        
    :ivar data_map: The value is a dict with the following keys:
        * cases - A dict that translates source field values
            to custom values.
        * default - An optional default value to use if the
            source attribute is not specified in a document.
        * name - the name of the document source field to add
            to this ``IndexField``
    :ivar data_trim_title: Trims common title words from a source
        document attribute when populating an ``IndexField``.
        This can be used to create an ``IndexField`` you can
        use for sorting.  The value is a dict with the following
        fields:
        * default - An optional default value.
        * language - an IETF RFC 4646 language code.
        * separator - The separator that follows the text to trim.
        * name - The name of the document source field to add.
    """

    ValidDataFunctions = ('Copy', 'TrimTitle', 'Map')

    def __init__(self):
        self.data_copy = {}
        self._data_function = self.ValidDataFunctions[0]
        self.data_map = {}
        self.data_trim_title = {}

    @property
    def data_function(self):
        return self._data_function

    @data_function.setter
    def data_function(self, value):
        if value not in self.ValidDataFunctions:
            valid = '|'.join(self.ValidDataFunctions)
            raise ValueError('data_function must be one of: %s' % valid)
        self._data_function = value



# Copyright (c) 2009 Chris Moyer http://coredumped.org/
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish, dis-
# tribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the fol-
# lowing conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS

#
# Elastic Load Balancer Tool
#
VERSION = "0.2"
usage = """%prog [options] [command]
Commands:
    list|ls                       List all Elastic Load Balancers
    delete    <name>              Delete ELB <name>
    get       <name>              Get all instances associated with <name>
    create    <name>              Create an ELB; -z and -l are required
    add       <name> <instances>  Add <instances> in ELB <name>
    remove|rm <name> <instances>  Remove <instances> from ELB <name>
    reap      <name>              Remove terminated instances from ELB <name>
    enable|en <name> <zone>       Enable Zone <zone> for ELB <name>
    disable   <name> <zone>       Disable Zone <zone> for ELB <name>
    addl      <name>              Add listeners (specified by -l) to the ELB
                                      <name>
    rml       <name> <port>       Remove Listener(s) specified by the port on
                                      the ELB <name>
"""


def find_elb(elb, name):
    try:
        elbs = elb.get_all_load_balancers(name)
    except boto.exception.BotoServerError as se:
        if se.code == 'LoadBalancerNotFound':
            elbs = []
        else:
            raise

    if len(elbs) < 1:
        print "No load balancer by the name of %s found" % name
        return None
    elif len(elbs) > 1:
        print "More than one elb matches %s?" % name
        return None

    # Should not happen
    if name not in elbs[0].name:
        print "No load balancer by the name of %s found" % name
        return None

    return elbs[0]


def list(elb):
    """List all ELBs"""
    print "%-20s %s" % ("Name", "DNS Name")
    print "-" * 80
    for b in elb.get_all_load_balancers():
        print "%-20s %s" % (b.name, b.dns_name)

def check_valid_region(conn, region):
    if conn is None:
        print 'Invalid region (%s)' % region
        sys.exit(1)

def get(elb, name):
    """Get details about ELB <name>"""

    b = find_elb(elb, name)
    if b:
        print "=" * 80
        print "Name: %s" % b.name
        print "DNS Name: %s" % b.dns_name
        if b.canonical_hosted_zone_name:
            chzn = b.canonical_hosted_zone_name
            print "Canonical hosted zone name: %s" % chzn
        if b.canonical_hosted_zone_name_id:
            chznid = b.canonical_hosted_zone_name_id
            print "Canonical hosted zone name id: %s" % chznid
        print

        print "Health Check: %s" % b.health_check
        print

        print "Listeners"
        print "---------"
        print "%-8s %-8s %s" % ("IN", "OUT", "PROTO")
        for l in b.listeners:
            print "%-8s %-8s %s" % (l[0], l[1], l[2])

        print

        print "  Zones  "
        print "---------"
        for z in b.availability_zones:
            print z

        print

        # Make map of all instance Id's to Name tags
        import boto
        from boto.compat.six import iteritems
        if not options.region:
            ec2 = boto.connect_ec2()
        else:
            ec2 = boto.ec2.connect_to_region(options.region)
            check_valid_region(ec2, options.region)

        instance_health = b.get_instance_health()
        instances = [state.instance_id for state in instance_health]

        names = dict((k,'') for k in instances)
        for i in ec2.get_only_instances():
            if i.id in instances:
                names[i.id] = i.tags.get('Name', '')

        name_column_width = max([4] + [len(v) for k,v in iteritems(names)]) + 2

        print "Instances"
        print "---------"
        print "%-12s %-15s %-*s %s" % ("ID",
                                       "STATE",
                                       name_column_width, "NAME",
                                       "DESCRIPTION")
        for state in instance_health:
            print "%-12s %-15s %-*s %s" % (state.instance_id,
                                           state.state,
                                           name_column_width, names[state.instance_id],
                                           state.description)

        print


def create(elb, name, zones, listeners):
    """Create an ELB named <name>"""
    l_list = []
    for l in listeners:
        l = l.split(",")
        if l[2] == 'HTTPS':
            l_list.append((int(l[0]), int(l[1]), l[2], l[3]))
        else:
            l_list.append((int(l[0]), int(l[1]), l[2]))

    b = elb.create_load_balancer(name, zones, l_list)
    return get(elb, name)


def delete(elb, name):
    """Delete this ELB"""
    b = find_elb(elb, name)
    if b:
        b.delete()
        print "Load Balancer %s deleted" % name


def add_instances(elb, name, instances):
    """Add <instance> to ELB <name>"""
    b = find_elb(elb, name)
    if b:
        b.register_instances(instances)
        return get(elb, name)


def remove_instances(elb, name, instances):
    """Remove instance from elb <name>"""
    b = find_elb(elb, name)
    if b:
        b.deregister_instances(instances)
        return get(elb, name)


def reap_instances(elb, name):
    """Remove terminated instances from elb <name>"""
    b = find_elb(elb, name)
    if b:
        for state in b.get_instance_health():
            if (state.state == 'OutOfService' and
                state.description == 'Instance is in terminated state.'):
                b.deregister_instances([state.instance_id])
        return get(elb, name)


def enable_zone(elb, name, zone):
    """Enable <zone> for elb"""
    b = find_elb(elb, name)
    if b:
        b.enable_zones([zone])
        return get(elb, name)


def disable_zone(elb, name, zone):
    """Disable <zone> for elb"""
    b = find_elb(elb, name)
    if b:
        b.disable_zones([zone])
        return get(elb, name)


def add_listener(elb, name, listeners):
    """Add listeners to a given load balancer"""
    l_list = []
    for l in listeners:
        l = l.split(",")
        l_list.append((int(l[0]), int(l[1]), l[2]))
    b = find_elb(elb, name)
    if b:
        b.create_listeners(l_list)
        return get(elb, name)


def rm_listener(elb, name, ports):
    """Remove listeners from a given load balancer"""
    b = find_elb(elb, name)
    if b:
        b.delete_listeners(ports)
        return get(elb, name)


if __name__ == "__main__":
    try:
        import readline
    except ImportError:
        pass
    import boto
    import sys
    from optparse import OptionParser
    from boto.mashups.iobject import IObject
    parser = OptionParser(version=VERSION, usage=usage)
    parser.add_option("-z", "--zone",
                      help="Operate on zone",
                      action="append", default=[], dest="zones")
    parser.add_option("-l", "--listener",
                      help="Specify Listener in,out,proto",
                      action="append", default=[], dest="listeners")
    parser.add_option("-r", "--region",
                      help="Region to connect to",
                      action="store", dest="region")

    (options, args) = parser.parse_args()

    if len(args) < 1:
        parser.print_help()
        sys.exit(1)

    if not options.region:
        elb = boto.connect_elb()
    else:
        import boto.ec2.elb
        elb = boto.ec2.elb.connect_to_region(options.region)
        check_valid_region(elb, options.region)

    print "%s" % (elb.region.endpoint)

    command = args[0].lower()
    if command in ("ls", "list"):
        list(elb)
    elif command == "get":
        get(elb, args[1])
    elif command == "create":
        if not options.listeners:
            print "-l option required for command create"
            sys.exit(1)
        if not options.zones:
            print "-z option required for command create"
            sys.exit(1)
        create(elb, args[1], options.zones, options.listeners)
    elif command == "delete":
        delete(elb, args[1])
    elif command in ("add", "put"):
        add_instances(elb, args[1], args[2:])
    elif command in ("rm", "remove"):
        remove_instances(elb, args[1], args[2:])
    elif command == "reap":
        reap_instances(elb, args[1])
    elif command in ("en", "enable"):
        enable_zone(elb, args[1], args[2])
    elif command == "disable":
        disable_zone(elb, args[1], args[2])
    elif command == "addl":
        if not options.listeners:
            print "-l option required for command addl"
            sys.exit(1)
        add_listener(elb, args[1], options.listeners)
    elif command == "rml":
        if not args[2:]:
            print "port required"
            sys.exit(2)
        rm_listener(elb, args[1], args[2:])
